package io.zipcoder;

import com.sun.xml.internal.fastinfoset.util.CharArray;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class ItemParser {

    private static final Logger log = Logger.getLogger( ItemParser.class.getName() );
    private int exceptionCount = 0;
    private Pattern patternAlpha = Pattern.compile("(?<=:)[^##\\s].*");
    private Pattern patternDate = Pattern.compile("\\d/\\d\\d/\\d\\d\\d\\d");

    public int getExceptionCount() {
        return exceptionCount-1;
    }

    public void setExceptionCount(int exceptionCount) {
        this.exceptionCount = exceptionCount;
    }

    public ArrayList<String> parseRawDataIntoStringArray(String rawData){
        String stringPattern = "##";
        ArrayList<String> response = splitStringWithRegexPattern(stringPattern , rawData);
        return response;
    }

    public String toLowerCase(String str){
            StringBuilder inputLineT = new StringBuilder(str);
            for(int i = 0 ; i < inputLineT.length() ; i++) {
                if(inputLineT.charAt(i) >= 65 && inputLineT.charAt(i) <=91)
                { inputLineT.setCharAt(i, (char)(inputLineT.charAt(i)+32)); }
            }
            return inputLineT.toString();
        }

    public String fixCookies(String str){
        String result ="";
        for (char c: str.toCharArray()) {
            if(c == '0'){result += "o";}
            else if (c != 0) result += c;
        }
        return result;
    }


    public Item parseStringIntoItem(String rawItem) throws ItemParseException{
        ArrayList<String> properties = findKeyValuePairsInRawItemData(toLowerCase(toLowerCase(rawItem)));
        Matcher mName = patternAlpha.matcher(properties.get(0));
        Matcher mPrice = patternAlpha.matcher(properties.get(1));
        Matcher mDescription = patternAlpha.matcher(properties.get(2));
        Matcher mExpiration = patternDate.matcher(properties.get(3));

    if (mName.find() && mDescription.find() && mPrice.find() && mExpiration.find()) {
        String name = fixCookies(mName.group());
        Double price = Double.parseDouble(mPrice.group());
        String type = mDescription.group();
        String expiration = mExpiration.group();
        return new Item(name, price, type, expiration);
    } else {
        throw new ItemParseException(); }
    }

    public ArrayList<Item> makeItemList(String rawData){
        ArrayList<String> data = parseRawDataIntoStringArray(rawData);
        ArrayList<Item> items = new ArrayList<Item>();
        int exceptionCount = 1;
        for (String parcel : data) {
            try {
                Item itemObj = parseStringIntoItem(parcel);
               items.add(itemObj);
            } catch (ItemParseException e){
                exceptionCount++;
                continue;
            }
        }
        setExceptionCount(exceptionCount);
        return items;
    }

    public Set<String> getUniqueNames(ArrayList<Item> list){
        Set<String> set = new HashSet<String>();
        for (Item item : list) {
            set.add(item.getName());
        }
            return set;
    }

    public int getCountName(String name, ArrayList<Item> itemList){
       int count = 0;
        for (Item item : itemList) {
            if (item.getName().equals(name)) count++;
        }
        return count;}

    public Set<Double> getUniquePrices(String name, ArrayList<Item> itemList){
        Set<Double> prices = new HashSet<Double>();
        for (Item item: itemList) {
            if(item.getName().equals(name)){
                prices.add(item.getPrice());
            }
        }
        return prices;
    }

    public int getCountPrice(Double price, String name, ArrayList<Item> itemList){
        int count = 0;
        for (Item item : itemList) {
            if (item.getName().equals(name) && item.getPrice().equals(price)) count++;
        }
        return count;
    }

    public void printOutput(String rawData){
       ArrayList<Item> itemList = makeItemList(rawData);
        Set<String> nameSet = getUniqueNames(itemList);
        for (String name : nameSet) {
            Set<Double> priceList = getUniquePrices(name, itemList);
            System.out.printf("%-13s %13s %n", "name: " + name, "seen: " + getCountName(name, itemList) + " times");
            System.out.printf("%-13s %13s %n", "=============", "=============");
            for (Double price : priceList) {
                System.out.printf("%-13s %13s %n", "price: " + price,"seen: " + getCountPrice(price, name, itemList) + " times");
                System.out.printf("%-13s %13s %n", "-------------", "-------------");
            }
            System.out.println();
        }
        System.out.printf("%-15s %15s %n","Errors", "seen: "+ getExceptionCount() + " times");
    }

    public ArrayList<String> findKeyValuePairsInRawItemData(String rawItem){
        String stringPattern = "[;|^!%@*]";
        ArrayList<String> response = splitStringWithRegexPattern(stringPattern , rawItem);
        return response;
    }

    private ArrayList<String> splitStringWithRegexPattern(String stringPattern, String inputString){
        return new ArrayList<String>(Arrays.asList(inputString.split(stringPattern)));
    }



}
